package lcsb.mapviewer.model.map;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.xml.bind.annotation.XmlRootElement;

import org.apache.log4j.Logger;

import lcsb.mapviewer.common.exception.InvalidArgumentException;

/**
 * Representation of a single species or reaction annotation. Miriam format is a
 * standard described <a href="http://www.ebi.ac.uk/miriam/main/">here</a>
 * 
 * @author Piotr Gawron
 * 
 */
@Entity
@Table(name = "miriam_data_table")
@org.hibernate.annotations.GenericGenerator(name = "test-increment-strategy", strategy = "increment")
@XmlRootElement
public class MiriamData implements Comparable<MiriamData>, Serializable {

	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger			 logger						= Logger.getLogger(MiriamData.class);

	/**
	 * 
	 */
	private static final long	 serialVersionUID	= 1L;

	/**
	 * Unique database identifier.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "idDb", unique = true, nullable = false)
	private int								 id;

	/**
	 * What is the connection between element and the annotation.
	 */
	private MiriamRelationType relationType			= null;

	/**
	 * Type of database.
	 */
	@Enumerated(EnumType.STRING)
	private MiriamType				 dataType;

	/**
	 * Resource identifier in the database.
	 */
	@Column(name = "resource")
	private String						 resource					= "";

	/**
	 * Default constructor.
	 */
	public MiriamData() {
		super();
	}

	/**
	 * Constructor that initialize the data by information from param.
	 * 
	 * @param md
	 *          original miriam data object
	 */
	public MiriamData(MiriamData md) {
		setRelationType(md.relationType);
		setDataType(md.dataType);
		setResource(md.resource);
	}

	/**
	 * Constructor that initialize the data by information from params.
	 * 
	 * @param relationType
	 *          {@link #relationType}
	 * @param mt
	 *          type of the miriam data (see: {@link MiriamType})
	 * @param resource
	 *          {@link #resource}
	 */
	public MiriamData(MiriamRelationType relationType, MiriamType mt, String resource) {
		if (mt == null) {
			throw new InvalidArgumentException("MiriamType cannot be null");
		}
		if (relationType == null) {
			throw new InvalidArgumentException("MiriamRelationType cannot be null");
		}
		setRelationType(relationType);
		setDataType(mt);
		setResource(resource);
	}

	/**
	 * Constructor that initialize the data by information from params.
	 * 
	 * @param mt
	 *          type of the miriam data (see: {@link MiriamType})
	 * @param resource
	 *          {@link #resource}
	 */
	public MiriamData(MiriamType mt, String resource) {
		this(MiriamRelationType.BQ_BIOL_IS_DESCRIBED_BY, mt, resource);
	}

	/**
	 * 
	 * @return {@link #resource}
	 */
	public String getResource() {
		return resource;
	}

	/**
	 * @param identifier
	 *          string represents identifier.
	 * @return {@link #resource}
	 */
	public static String getIdFromIdentifier(String identifier) {
		if (identifier != null) {
			int index = identifier.indexOf(":");
			if (index != -1) {
				identifier = identifier.trim().substring(index + 1);
			}
		}
		return identifier;
	}

	/**
	 * Sets new {@link #resource}.
	 * 
	 * @param resource
	 *          new {@link #resource}
	 */
	public void setResource(String resource) {
		if (resource == null) {
			this.resource = null;
		} else {
			this.resource = resource.replace("%3A", ":");
		}
	}

	@Override
	public boolean equals(Object aThat) {
		// check for self-comparison
		if (this == aThat) {
			return true;
		}

		// use instanceof instead of getClass here for two reasons
		// 1. if need be, it can match any supertype, and not just one class;
		// 2. it renders an explict check for "that == null" redundant, since
		// it does the check for null already - "null instanceof [type]" always
		// returns false. (See Effective Java by Joshua Bloch.)
		if (!(aThat instanceof MiriamData)) {
			return false;
		}
		// Alternative to the above line :
		// if ( aThat == null || aThat.getClass() != this.getClass() ) return false;

		// cast to native object is now safe
		MiriamData that = (MiriamData) aThat;

		// now a proper field-by-field evaluation can be made
		return this.compareTo(that) == 0;
	}

	@Override
	public int hashCode() {
		return (dataType + resource).hashCode();
	}

	@Override
	public int compareTo(MiriamData other) {
		return ((dataType + ":" + resource).toLowerCase().compareTo((other.dataType + ":" + other.resource).toLowerCase()));
	}

	/**
	 * @return the id
	 * @see #id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id
	 *          the id to set
	 * @see #id
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * @return the dataType
	 * @see #dataType
	 */
	public MiriamType getDataType() {
		return dataType;
	}

	/**
	 * @param dataType
	 *          the dataType to set
	 * @see #dataType
	 */
	public void setDataType(MiriamType dataType) {
		this.dataType = dataType;
	}

	/**
	 * @return the relationType
	 * @see #relationType
	 */
	public MiriamRelationType getRelationType() {
		return relationType;
	}

	/**
	 * @param relationType
	 *          the relationType to set
	 * @see #relationType
	 */
	public void setRelationType(MiriamRelationType relationType) {
		this.relationType = relationType;
	}

	@Override
	public String toString() {
		if (relationType != null) {
			return "[" + relationType.getStringRepresentation() + "] " + dataType + ":" + resource;
		} else {
			return "[UNKNOWN] " + dataType + ":" + resource;

		}
	}
}
