package lcsb.mapviewer.model.map.species.field;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.apache.log4j.Logger;
import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;

import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.species.AntisenseRna;

/**
 * This structure contains information about antisense rna region (rna fragment
 * of interest) for a specific {@link AntisenseRna}.
 * 
 * @author Piotr Gawron
 * 
 */
@Entity
@Table(name = "antisense_rna_region_table")
@org.hibernate.annotations.GenericGenerator(name = "test-increment-strategy", strategy = "increment")
public class AntisenseRnaRegion implements Serializable, ElementModification {

	/**
	 * 
	 */
	private static final long			 serialVersionUID			= 1L;

	/**
	 * Default size of the object (in graphical representation).
	 */
	private static final double		 DEFAULT_SIZE					= 0.1;

	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger					 logger								= Logger.getLogger(AntisenseRnaRegion.class);

	/**
	 * Unique identifier in the database.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "iddb", unique = true, nullable = false)
	private int										 id;

	/**
	 * String identifier of the element. Unique in the model.
	 */
	@Column(name = "idantisensernaregion")
	private String								 idAntisenseRnaRegion	= "";

	/**
	 * Name of the region.
	 */
	@Column(name = "name")
	private String								 name									= "";

	/**
	 * Defines a state of the region (for instance ubiquitinated etc).
	 * 
	 * @see ModificationState
	 */
	@Column(name = "state")
	private ModificationState			 state								= null;

	/**
	 * Type of the region in the rna. There are three possible values:
	 * <ul>
	 * <li>Coding region,</li>
	 * <li>Protein binding domain,</li>
	 * <li>Modification site.</li>
	 * </ul>
	 */
	@Column(name = "type")
	private AntisenseRnaRegionType type;

	/**
	 * Size of the region in the graphic representation.
	 */
	@Column(name = "size")
	private double								 size									= DEFAULT_SIZE;

	/**
	 * Position on the species in graphic representation.
	 */
	@Column(name = "pos")
	private Double								 pos;

	/**
	 * Defines a species where the region belongs to.
	 */
	@Cascade({ CascadeType.ALL })
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "idSpeciesDb")
	private AntisenseRna					 species;

	/**
	 * Default constructor.
	 */
	public AntisenseRnaRegion() {

	}

	/**
	 * Constructor that initialize object with the data from the parameter.
	 * 
	 * @param original
	 *          object from which we initialize data
	 */
	public AntisenseRnaRegion(AntisenseRnaRegion original) {
		this.idAntisenseRnaRegion = original.idAntisenseRnaRegion;
		this.name = original.name;
		this.size = original.size;
		this.pos = original.pos;
		this.type = original.type;
	}

	/**
	 * Creates a copy of current object.
	 * 
	 * @return copy of the object
	 */
	public AntisenseRnaRegion copy() {
		if (this.getClass() == AntisenseRnaRegion.class) {
			return new AntisenseRnaRegion(this);
		} else {
			throw new NotImplementedException("Method copy() should be overriden in class " + this.getClass());
		}

	}

	/**
	 * Sets size from the string.
	 * 
	 * @param text
	 *          size to parse and set
	 * @see #size
	 */
	public void setSize(String text) {
		try {
			size = Double.parseDouble(text);
		} catch (NumberFormatException e) {
			throw new InvalidArgumentException("Invalid angle: " + text, e);
		}
	}

	/**
	 * Sets position from the string.
	 * 
	 * @param text
	 *          position to parse and set
	 * @see #pos
	 */
	public void setPos(String text) {
		try {
			pos = Double.parseDouble(text);
		} catch (NumberFormatException e) {
			throw new InvalidArgumentException("Invalid pos: " + text, e);
		}
	}

	/**
	 * Update data in this object from parameter (only if values in parameter
	 * object are valid).
	 * 
	 * @param mr
	 *          object from which we are updating data
	 */
	public void update(AntisenseRnaRegion mr) {
		if (this.idAntisenseRnaRegion != null && !this.idAntisenseRnaRegion.equals("") && !this.idAntisenseRnaRegion.equals(mr.getIdAntisenseRnaRegion())) {
			throw new InvalidArgumentException("Cannot update from mr with different id");
		}
		this.size = mr.getSize();
		if (mr.getState() != null) {
			this.state = mr.getState();
		}
		if (mr.getName() != null) {
			this.name = mr.getName();
		}
		if (mr.getPos() != null) {
			this.setPos(mr.getPos());
		}

	}

	@Override
	public String toString() {
		String result = getIdAntisenseRnaRegion() + "," + getName() + "," + getType() + "," + getPos() + "," + getSize() + ",";
		return result;
	}

	/**
	 * @return the idAntisenseRnaRegion
	 * @see #idAntisenseRnaRegion
	 */
	public String getIdAntisenseRnaRegion() {
		return idAntisenseRnaRegion;
	}

	/**
	 * @param idAntisenseRnaRegion
	 *          the id to set
	 * @see #idAntisenseRnaRegion
	 */
	public void setIdAntisenseRnaRegion(String idAntisenseRnaRegion) {
		this.idAntisenseRnaRegion = idAntisenseRnaRegion;
	}

	/**
	 * @return the name
	 * @see #name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *          the name to set
	 * @see #name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the state
	 * @see #state
	 */
	public ModificationState getState() {
		return state;
	}

	/**
	 * @param state
	 *          the state to set
	 * @see #state
	 */
	public void setState(ModificationState state) {
		this.state = state;
	}

	/**
	 * @return the type
	 * @see #type
	 */
	public AntisenseRnaRegionType getType() {
		return type;
	}

	/**
	 * @param type
	 *          the type to set
	 * @see #type
	 */
	public void setType(AntisenseRnaRegionType type) {
		this.type = type;
	}

	/**
	 * @return the size
	 * @see #size
	 */
	public double getSize() {
		return size;
	}

	/**
	 * @param size
	 *          the size to set
	 * @see #size
	 */
	public void setSize(double size) {
		this.size = size;
	}

	/**
	 * @return the pos
	 * @see #pos
	 */
	public Double getPos() {
		return pos;
	}

	/**
	 * @param pos
	 *          the pos to set
	 * @see #pos
	 */
	public void setPos(Double pos) {
		this.pos = pos;
	}

	/**
	 * @return the species
	 * @see #species
	 */
	public AntisenseRna getSpecies() {
		return species;
	}

	/**
	 * @param species
	 *          the species to set
	 * @see #species
	 */
	public void setSpecies(AntisenseRna species) {
		this.species = species;
	}

	/**
	 * @return the id
	 * @see #id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id
	 *          the id to set
	 * @see #id
	 */
	public void setId(int id) {
		this.id = id;
	}

}
