package lcsb.mapviewer.model.map.species.field;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.apache.log4j.Logger;

import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.model.map.species.Rna;

/**
 * This structure contains information about rna region (rna fragment of
 * interest) for a specific {@link Rna}.
 * 
 * @author Piotr Gawron
 * 
 */
@Entity
@Table(name = "rna_region_table")
@org.hibernate.annotations.GenericGenerator(name = "test-increment-strategy", strategy = "increment")
public class RnaRegion implements Serializable, ElementModification {

	/**
	 * 
	 */
	private static final long		serialVersionUID = 1L;

	/**
	 * Default {@link #size} of the region.
	 */
	private static final double	DEFAULT_SIZE		 = 0.1;

	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger				logger					 = Logger.getLogger(RnaRegion.class.getName());

	/**
	 * Unique identifier in the database.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "iddb", unique = true, nullable = false)
	private int									id;

	/**
	 * Identifier of the region. Unique in the
	 * {@link lcsb.mapviewer.model.map.model.Model}.
	 */
	@Column(name = "idrnaregion")
	private String							idRnaRegion			 = "";
	/**
	 * Type of the region in the rna. There are three possible values:
	 * <ul>
	 * <li>Coding region,</li>
	 * <li>Protein binding domain,</li>
	 * <li>Modification site.</li>
	 * </ul>
	 */

	@Column(name = "type")
	private String							type						 = "";

	/**
	 * Defines a state of the region (for instance ubiquitinated etc).
	 * 
	 * @see ModificationState
	 */
	@Column(name = "state")
	private ModificationState		state						 = null;

	/**
	 * Name of the region.
	 */
	@Column(name = "name")
	private String							name						 = "";

	/**
	 * Size of the region in the graphic representation.
	 */
	@Column(name = "size")
	private double							size						 = DEFAULT_SIZE;

	/**
	 * Position on the species in graphic representation.
	 */
	@Column(name = "pos")
	private Double							pos;

	/**
	 * Defines a species where the region belongs to.
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "idSpeciesDb")
	private Rna									species;

	/**
	 * Default constructor.
	 */
	public RnaRegion() {

	}

	/**
	 * Creates object with the data taken from paramter region.
	 * 
	 * @param mr
	 *          original {@link RnaRegion}
	 */
	public RnaRegion(RnaRegion mr) {
		this.id = mr.id;
		this.idRnaRegion = mr.idRnaRegion;
		this.size = mr.size;
		setPos(mr.getPos());
		this.type = mr.type;
		this.state = mr.state;
		this.name = mr.name;
	}

	/**
	 * Sets {@link #size}.
	 * 
	 * @param text
	 *          new size value in string format
	 */
	public void setSize(String text) {
		try {
			size = Double.parseDouble(text);
		} catch (NumberFormatException e) {
			throw new InvalidArgumentException("Invalid angle: " + text, e);
		}

	}

	/**
	 * Sets {@link #pos}.
	 * 
	 * @param text
	 *          new {@link #pos} value in string format
	 */
	public void setPos(String text) {
		try {
			setPos(Double.parseDouble(text));
		} catch (NumberFormatException e) {
			throw new InvalidArgumentException("Invalid pos: " + text, e);
		}

	}

	/**
	 * Updates fields in the object with the data given in the parameter rna
	 * region.
	 * 
	 * @param mr
	 *          {@link RnaRegion} from which data will be used to update
	 */
	public void update(RnaRegion mr) {
		if (this.idRnaRegion != null && !this.idRnaRegion.equals("") && !this.idRnaRegion.equals(mr.getIdRnaRegion())) {
			throw new InvalidArgumentException("Cannot update from mr with different id");
		}
		this.size = mr.getSize();
		if (mr.getState() != null) {
			this.state = mr.getState();
		}
		if (mr.getName() != null) {
			this.name = mr.getName();
		}
		if (mr.getPos() != null) {
			this.setPos(mr.getPos());
		}

	}

	@Override
	public String toString() {
		String result = "" + getIdRnaRegion() + "," + getType() + "," + getPos() + "," + getSize() + "," + getState() + ",";
		return result;

	}

	/**
	 * @return the idRnaRegion
	 * @see #idRnaRegion
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id
	 *          the idRnaRegion to set
	 * @see #id
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * @return the id
	 * @see #id
	 */
	public String getIdRnaRegion() {
		return idRnaRegion;
	}

	/**
	 * @param id
	 *          the id to set
	 * @see #id
	 */
	public void setIdRnaRegion(String id) {
		this.idRnaRegion = id;
	}

	/**
	 * @return the type
	 * @see #type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @param type
	 *          the type to set
	 * @see #type
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * @return the state
	 * @see #state
	 */
	public ModificationState getState() {
		return state;
	}

	/**
	 * @param state
	 *          the state to set
	 * @see #state
	 */
	public void setState(ModificationState state) {
		this.state = state;
	}

	/**
	 * @return the name
	 * @see #name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *          the name to set
	 * @see #name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the size
	 * @see #size
	 */
	public double getSize() {
		return size;
	}

	/**
	 * @param size
	 *          the size to set
	 * @see #size
	 */
	public void setSize(double size) {
		this.size = size;
	}

	/**
	 * @return the pos
	 * @see #pos
	 */
	public Double getPos() {
		return pos;
	}

	/**
	 * @param pos
	 *          the pos to set
	 * @see #pos
	 */
	public void setPos(Double pos) {
		this.pos = pos;
	}

	/**
	 * @return the species
	 * @see #species
	 */
	public Rna getSpecies() {
		return species;
	}

	/**
	 * @param species
	 *          the species to set
	 * @see #species
	 */
	public void setSpecies(Rna species) {
		this.species = species;
	}

}
