package lcsb.mapviewer.model.user;

/**
 * This enumerate defines all possible configuration parameter that are
 * configurable by the user.
 * 
 * @author Piotr Gawron
 * 
 */
public enum ConfigurationElementType {

  /**
   * Email address used for sending email from the system.
   */
  EMAIL_ADDRESS("E-mail address", "your.account@domain.com", ConfigurationElementEditType.EMAIL, true), //

  /**
   * Login for the email account.
   */
  EMAIL_LOGIN("E-mail server login", "your@login", ConfigurationElementEditType.STRING, true), //

  /**
   * Password for the email account.
   */
  EMAIL_PASSWORD("E-mail server password", "email.secret.password", ConfigurationElementEditType.PASSWORD, true), //

  /**
   * Address of the IMAP server.
   */
  EMAIL_IMAP_SERVER("IMAP server", "your.imap.domain.com", ConfigurationElementEditType.STRING, true), //

  /**
   * Address of the SMTP server.
   */
  EMAIL_SMTP_SERVER("SMTP server", "your.smtp.domain.com", ConfigurationElementEditType.STRING, true), //

  /**
   * Port used for SMTP connection (sending e-mails).
   */
  EMAIL_SMTP_PORT("SMTP port", "25", ConfigurationElementEditType.INTEGER, true), //

  /**
   * Default map that should be presented if no map is selected by user side.
   */
  DEFAULT_MAP("Default Project Id", "empty", ConfigurationElementEditType.STRING, false), //

  /**
   * Logo presented in the system.
   */
  LOGO_IMG("Logo icon", "udl.png", ConfigurationElementEditType.URL, false), //

  /**
   * Address connected to the logo.
   */
  LOGO_LINK("Logo link (after click)", "http://wwwen.uni.lu/", ConfigurationElementEditType.URL, false), //

  /**
   * Maximum distance (in pixels) that is allowed during finding closest element
   * on the map.
   */
  SEARCH_DISTANCE("Max distance for clicking on element (px)", "10", ConfigurationElementEditType.DOUBLE, false),

  /**
   * Email used for requesting an account (in client side).
   */
  REQUEST_ACCOUNT_EMAIL("Email used for requesting an account", "your.email@domain.com",
      ConfigurationElementEditType.EMAIL, false),

  /**
   * Max number of results in search box.
   */
  SEARCH_RESULT_NUMBER("Max number of results in search box. ", "100", ConfigurationElementEditType.INTEGER, false),

  /**
   * Google Analytics tracking ID used for statistics. This tracking ID should
   * look like "UA-000000-01". More information about tracking ID can be found
   * <a href="https://support.google.com/analytics/answer/1032385?hl=en"> here
   * </a>.
   */
  GOOGLE_ANALYTICS_IDENTIFIER("Google Analytics tracking ID used for statistics", "",
      ConfigurationElementEditType.STRING, false),

  /**
   * Description of the logo presented in the system.
   */
  LOGO_TEXT("Logo description", "University of Luxembourg", ConfigurationElementEditType.STRING, false),

  /**
   * Domain allowed to connect via x-frame technology.
   */
  X_FRAME_DOMAIN("Domain allowed to connect via x-frame technology", "", ConfigurationElementEditType.URL, false),

  /**
   * Relative directory (in webapps folder) where big files will be stored.
   */
  BIG_FILE_STORAGE_DIR("Path to store big files", "minerva-big/", ConfigurationElementEditType.STRING, false),

  /**
   * File where legend 1/4 is stored.
   */
  LEGEND_FILE_1("Legend 1 image file", "resources/images/legend_a.png", ConfigurationElementEditType.URL, false),

  /**
   * File where legend 2/4 is stored.
   */
  LEGEND_FILE_2("Legend 2 image file", "resources/images/legend_b.png", ConfigurationElementEditType.URL, false),

  /**
   * File where legend 3/4 is stored.
   */
  LEGEND_FILE_3("Legend 3 image file", "resources/images/legend_c.png", ConfigurationElementEditType.URL, false),

  /**
   * File where legend 4/4 is stored.
   */
  LEGEND_FILE_4("Legend 4 image file", "resources/images/legend_d.png", ConfigurationElementEditType.URL, false),

  /**
   * File where legend 4/4 is stored.
   */
  USER_MANUAL_FILE("User manual file", "resources/other/user_guide.pdf", ConfigurationElementEditType.URL, false),

  /**
   * Color used for negative overlay values.
   */
  MIN_COLOR_VAL("Overlay color for negative values", "FF0000", ConfigurationElementEditType.COLOR, false),

  /**
   * Color used for positive overlay values.
   */
  MAX_COLOR_VAL("Overlay color for postive values", "0000FF", ConfigurationElementEditType.COLOR, false),

  /**
   * Color used for undefined overlay values.
   */
  SIMPLE_COLOR_VAL("Overlay color when no values are defined", "00FF00", ConfigurationElementEditType.COLOR, false),

  /**
   * Color used for 0 overlay value.
   */
  NEUTRAL_COLOR_VAL("Overlay color for value=0", "FFFFFF", ConfigurationElementEditType.COLOR, false),

  /**
   * Opacity of data overlay objects in the frontend.
   */
  OVERLAY_OPACITY("Opacity used when drwaing data overlays (value between 0.0-1.0)", "0.8",
      ConfigurationElementEditType.DOUBLE, false),

  /**
   * Default content of the email when requesting for an account in the system.
   */
  REQUEST_ACCOUNT_DEFAULT_CONTENT("Email content used for requesting an account",
      "Dear Disease map team,\nI would like to request an account in the system.\nKind regards",
      ConfigurationElementEditType.TEXT, false),

  DEFAULT_VIEW_PROJECT("Default user privilege for: " + PrivilegeType.VIEW_PROJECT.getCommonName(), "true",
      ConfigurationElementEditType.BOOLEAN, false),

  DEFAULT_EDIT_COMMENTS_PROJECT("Default user privilege for: " + PrivilegeType.EDIT_COMMENTS_PROJECT.getCommonName(),
      "false", ConfigurationElementEditType.BOOLEAN, false),

  DEFAULT_LAYOUT_MANAGEMENT("Default user privilege for: " + PrivilegeType.LAYOUT_MANAGEMENT.getCommonName(), "false",
      ConfigurationElementEditType.BOOLEAN, false),

  ;

  /**
   * Default value of the configuration parameter (it will be used only when value
   * doesn't exist in the DAO).
   */
  private String defaultValue = "";

  /**
   * Common name used for visualization (query user).
   */
  private String commonName = "";

  /**
   * How we want to edit specific parameter.
   */
  private ConfigurationElementEditType editType = null;

  private boolean serverSide = true;

  /**
   * Default constructor.
   * 
   * @param commonName
   *          common name used for this parameter
   * @param editType
   *          type defining how we want to edit this configuration parameter
   * @param defaultVal
   *          default value assigned to this parameter
   */
  ConfigurationElementType(String commonName, String defaultVal, ConfigurationElementEditType editType,
      boolean serverSide) {
    this.defaultValue = defaultVal;
    this.commonName = commonName;
    this.editType = editType;
    this.serverSide = true;
  }

  /**
   * @return the defaultValue
   * @see #defaultValue
   */
  public String getDefaultValue() {
    return defaultValue;
  }

  /**
   * @return the commonName
   * @see #commonName
   */
  public String getCommonName() {
    return commonName;
  }

  /**
   * @return the editType
   * @see #editType
   */
  public ConfigurationElementEditType getEditType() {
    return editType;
  }

  /**
   * @return the serverSide
   * @see #serverSide
   */
  public boolean isServerSide() {
    return serverSide;
  }

  /**
   * @param serverSide
   *          the serverSide to set
   * @see #serverSide
   */
  public void setServerSide(boolean serverSide) {
    this.serverSide = serverSide;
  }

}
